//
// Copyright (C) 2013-2016  Taiyou Sakashita, Hiroko Yamada
//
// Pick up Parameter in Particular Latitude Longitude 2015/08/10
// Modified for using Julidan day 2016/07/01
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <time.h>
#include <math.h>
#include "bin.c"
#include "jday.c"


// Length of the Data Column/Line Direction
int X;
int Y;

// Year/Month/Day/Hour/Minute
int YYYY;
int MM;
int DD;
int HH;
int MN;

// UTC Year/Month/Day/Hour
int UYYYY;
int UMM;
int UDD;
int UHH;
int UMN;

// Picked Pixel in X/Y Direction
int Pick_Pix[2+1];

// Time of Picked Data
int Pick_Time_UTC[6+1] ;
int Pick_Time_Local[6+1];

// Starndard Latitude/Longitude Data
float *STN_LAT;
float *STN_LNG;

// Solar Zenith Angle
double SZA;


// Use for Calculate SZA
double a1 = 0.000075;
double a2 = 0.001868;
double a3 = 0.032077;
double a4 = 0.014615;
double a5 = 0.040849;

double b1 = 0.006918;
double b2 = 0.399912;
double b3 = 0.070257;
double b4 = 0.006758;
double b5 = 0.000907;
double b6 = 0.002697;
double b7 = 0.001480;

// File Type
char ftype[256];

// Time File Type
char ttype[256];

// Directory Name to Read Original Data
char input_DIR[512];

// Directory Name to Write Extract Parameter
char Pickup_DIR[512];

// Directory Name to Read Standart Latitude/Longitude Data
char SL_DIR[512];

// Filename of Standard Latitude
char LATname[512];

// Filename of Standard Longitude
char LNGname[512];

// TIME DIFF  & Time Type
double TIME_DIFF; // int->doble for India 5.5H
char TIME_Code[512];

//jd_UTC_PickUP
double jd_UTC_PickUP;


float extract(double LAT, double LNG);
void search_extract_pixel(double LAT, double LNG, int *ANS_Pixel);
void time_check(double LAT, double LNG);
void shift_time(int Year, int Month, int Day, int Hour, int Minute, double Shift_HH, int *Shift_Time);
double cal_sza(int Year, int Month, int Day, int Hour, int Minute, double LAT, double LNG);

void mjd2utc(float Time, int *Pick_Time);
int get_lastday(int Year, int Month);
double date2doy(int Year, int Month, int Day, int Hour, int Minute);

void date_errorcheck();
void initialization();
void read_formatfile();
void read_standard_latlng();
void write_txt(char *filename, float *DD);



int main(int argc, char *argv[])
{


  // Filename to Write Extracted Data
  char wfilename[512];

  // Latitude/Longitude
  double LAT;
  double LNG;

  // Extracted parameter
  float Extract_param[3+1];


  // Year/Month/Day/Hour/Minute
  YYYY = atoi(argv[1]);
  MM = atoi(argv[2]);
  DD = atoi(argv[3]);
  HH = atoi(argv[4]);
  MN = atoi(argv[5]);

  LAT = atof(argv[6]);   //ascii->double
  LNG = atof(argv[7]);   //ascii->double

        struct stat  st;
        char fname[20];
        char str1[20] = "output/";
        sprintf(fname,"%s%d%02d",str1,YYYY,MM);
        
        if (stat(fname, &st) != 0){
            printf("%s dosen't exist\n", fname);
            if(mkdir(fname, 0777)==0){
                printf("Succeeded to create a directory\n");
                   }else{
                printf("Failed to create a directory\n");
            }
        }
        else {
                switch (st.st_mode & S_IFMT) {
                        case S_IFREG:
                                printf("%s is a file\n", fname);
                                break;
                        case S_IFDIR:
                                printf("%s is a dir\n", fname);
                                break;
                        default:
                                printf("%s isn't a file or dir\n", fname);
                        break;
                }
        }

        
  date_errorcheck();
  initialization();


  // Extract
  Extract_param[1] = LAT;
  Extract_param[2] = LNG;
  Extract_param[3] = extract(LAT, LNG);

  sprintf(wfilename, "%s%d%02d/%d%02d%02d%02d%02d.%s%s.pick.txt", Pickup_DIR, YYYY, MM, YYYY, MM, DD, HH, MN, TIME_Code, ftype);
 
 // Calculation Pick_Time_local
  double jd_utctolocal = jd_UTC_PickUP + TIME_DIFF/24.0;
  jd2ymdhn(jd_utctolocal,Pick_Time_Local);        

  // Write Extracted Data
  write_txt(wfilename, Extract_param);

  free(STN_LAT);
  free(STN_LNG);

  return 0;

}//main


float extract(double LAT, double LNG)
{

  // Answer
  float ANS;

  // Data Read
  float *DR;

  // Filename to Read Original Data
  char filename[512];

  struct stat st;


  if((DR = (float *)malloc(sizeof(float)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed.\n"); exit(1);}

// UTC Time
  int UTime[6+1];
  double jd_UTC = 0;

  double jd_jul_ymdhn=TransDateToJD(YYYY,MM,DD,HH,MN,0); //1.Local Time to JD(local)
  jd_UTC= jd_jul_ymdhn - TIME_DIFF/24.0;        //  2.JD(local) to JD(UTC)

  jd2ymdhn(jd_UTC,UTime);//  3.JD(UTC) -> Calendar(UTC)

  UYYYY = UTime[1];
  UMM = UTime[2];
  UDD = UTime[3];
  UHH = UTime[4];
  UMN = UTime[5];

  sprintf(filename, "%s%d%02d/%d%02d%02d%02d%02d%s", input_DIR, YYYY, MM, UYYYY, UMM, UDD, UHH, UMN, ftype);

  if(stat(filename, &st) != 0)
  {
     printf("%s is not found\n", filename);
     exit(1);

  }//fi

  search_extract_pixel(LAT, LNG, Pick_Pix);
  
  time_check(LAT, LNG);
  
  
  if(SZA < 90)
  {
    bin2f(filename, X, Y, DR);    
    ANS = (float)DR[X*(Pick_Pix[2]-1)+Pick_Pix[1]];
    
  }else{
    
    ANS = 0;
    
  }//fi


  free(DR);


  return ANS;


}//extract


void search_extract_pixel(double LAT, double LNG, int *ANS_Pixel)
{
  
  int I,J;

  // Minimum Distance
  double MIN_DIS = 10000;

  // Distance of Column/Line Direction
  double DIS_X;
  double DIS_Y;

  // Minimum Distance Pixel Number of Column/Line Direction
  int MIN_X = 1;
  int MIN_Y = 1;
  
  // Add T.Sakashita
  double DIS_Border;
  
  if(X<Y)
  {        
    DIS_Border = (STN_LNG[X*(1-1)+2]-STN_LNG[X*(1-1)+1])*1.5;
                             
  }else{
  
    DIS_Border = (STN_LAT[X*(1-1)+1]-STN_LAT[X*(2-1)+1])*1.5;
    
  }//fi

  
  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        DIS_X = pow(LNG - STN_LNG[X*(I-1)+J], 2);
        DIS_Y = pow(LAT - STN_LAT[X*(I-1)+J], 2);

        if( sqrt(DIS_X + DIS_Y) < MIN_DIS) 
        {
           MIN_DIS = sqrt(DIS_X + DIS_Y);
           MIN_X = J;
           MIN_Y = I;

        }//fi

    }//J
  }//I
  
  
  if(MIN_DIS > DIS_Border)
  {
     printf("%f, %f is out of range\n", LAT, LNG);
     exit(1);

  }//fi
  
  printf("Pick up %f, %f\n", LAT, LNG);
  
  ANS_Pixel[1] = MIN_X;
  ANS_Pixel[2] = MIN_Y;
   
  return;
  
  
}//search_extract_pixel


void time_check(double LAT, double LNG)
{

  // Local Time
  int LTime[5+1];

  // Local Year/Month/Day/Hour/Minute/Second
  float *Time;

  // Filenama of Time Data
  char Timename[512];


  if((Time = (float *)malloc(sizeof(float)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed.\n"); exit(1);}
  sprintf(Timename, "%s%d%02d/%d%02d%02d%02d%02d%s", input_DIR, YYYY, MM, UYYYY, UMM, UDD, UHH, UMN, ttype);

  bin2f(Timename, X, Y, Time);

  mjd2utc(Time[(Pick_Pix[2]-1)*X+Pick_Pix[1]], Pick_Time_UTC);
  
  SZA = cal_sza(Pick_Time_UTC[1], Pick_Time_UTC[2], Pick_Time_UTC[3], Pick_Time_UTC[4], Pick_Time_UTC[5], (double)LAT, (double)LNG);
  
  if(SZA >= 90)
  {
    printf("Solar Zenith Angle = %f is more than 90 Degrees\n", SZA);
    
  }//fi


  free(Time);


  return;


}//time_check


void mjd2utc(float Time, int *UTC_Time)
{

  int I,J;

  int A;
  int B;

  // Modified Julian Day
  int MJD;
  double mjd;

  double jd;
  int aa;
  int bb;
  int cc;
  int ee;
  int ff;
  int gg;
  int hh;
  double ii;

  // Use for Calculate Year/Month/Day
  int Use_YYYY;
  int Use_MM;
  int Use_DD;

  // Picked Year/Month/Day/Hour/Minute/Second
  int PYYYY;
  int PMM;
  int PDD;
  int PHH;
  int PMN;
  int PSC;


  if(MM==1 || MM==2)
  {
     Use_YYYY = YYYY-1;
     Use_MM   = MM+12;

  }else{

     Use_YYYY = YYYY;
     Use_MM = MM;

  }//fi

  Use_DD = DD;

  A = (int)(Use_YYYY/100);
  B = 2-A + (int)(A/4);

  MJD = (int)(round((int)(365.25*(Use_YYYY+4716)) + (int)(30.6001*(Use_MM+1))+ Use_DD + B -1524.5)-2400000.5);
  mjd = MJD + (double)Time;

  jd = mjd + 2400000.5;
  jd_UTC_PickUP = jd;

  aa=(int)(jd+68569.5);
  bb=(int)(aa/36524.25);
  cc=aa-(int)(36524.25*bb+0.75);
  ee=(int)((cc+1)/365.25025);
  ff=cc-(int)(365.25*ee)+31;
  gg=(int)(ff/30.59);
  hh=(int)(gg/11);
  ii=jd+0.5-(int)(jd+0.5);

  PYYYY=100*(bb-49)+ee+hh;
  PMM=gg-12*hh+2;
  PDD=ff-(int)(30.59*gg);
  PHH=(int)(24*ii);
  PMN=(int)(1440*(ii-PHH/24.0));
  PSC=(int)(86400*(ii-PHH/24.0-PMN/1440.0));

  UTC_Time[1]=PYYYY;
  UTC_Time[2]=PMM;
  UTC_Time[3]=PDD;
  UTC_Time[4]=PHH;
  UTC_Time[5]=PMN;
  UTC_Time[6]=PSC;


  return;


}//mjd2utc


void shift_time(int Year, int Month, int Day, int Hour, int Minute, double Shift_HH, int *Shift_Time)
{

  time_t timer;
  struct tm base;
  struct tm *shift;

  base.tm_year = Year-1900;
  base.tm_mon  = Month-1;
  base.tm_mday = Day;
  base.tm_hour = Hour+Shift_HH;
  base.tm_min  = Minute;
  base.tm_sec  = 0;

  timer = mktime(&base);

  shift = localtime(&timer);

  Shift_Time[1] = shift->tm_year+1900;
  Shift_Time[2] = shift->tm_mon+1;
  Shift_Time[3] = shift->tm_mday;
  Shift_Time[4] = shift->tm_hour;
  Shift_Time[5] = shift->tm_min;


  return;


}//shift_time


int get_lastday(int Year, int Month)
{

  // Last Day of the Month
  int Last_DD;


  time_t timer;
  struct tm base;
  struct tm *shift;

  base.tm_year = Year-1900;
  base.tm_mon  = Month;
  base.tm_mday = 1;
  base.tm_hour = 0-1;
  base.tm_min  = 0;
  base.tm_sec  = 0;

  timer = mktime(&base);

  shift = localtime(&timer);

  Last_DD = shift->tm_mday;


  return Last_DD;


}//get_lastday


double date2doy(int Year, int Month, int Day, int Hour, int Minute)
{
  
    // Day of Year
    int DOY;
    
    time_t timer;
    struct tm base;
    struct tm *check;
    
    base.tm_year = Year-1900;
    base.tm_mon = Month-1;
    base.tm_mday = Day;
    base.tm_hour = Hour;
    base.tm_min  = Minute;
    base.tm_sec  = 0;
    
    timer = mktime(&base);
    
    check = localtime(&timer);
    
    DOY = check->tm_yday + 1;
   
    
    return DOY;
    
    
}//date2md


double cal_sza(int Year, int Month, int Day, int Hour, int Minute, double LAT, double LNG)
{
  
  // Day of Year
  int DOY;
  
  // Local Time
  int Local_Time[5+1];
  
  // Greenwith Meridian Time
  double UTC_Time;
  
  // Solar Declination
  double DEC;
  
  double A;
  double B;
  
  // Hour Angle
  double t;
  
  // Equqtion of Time
  double ET;
  
  // Mean Solar Time
  double MST;
  
  // True Solar Time
  double TST;
  
  // cos(Solar Zenith Angle)
  double SZA_cos;
  
  // Solar Zenith Angle
  double ANS;
  

  // Calculate HH + MN
  UTC_Time = Hour + (double)Minute / 60;
  
  // Calculate Local Time
  shift_time(Year, Month, Day, Hour, Minute, TIME_DIFF, Local_Time);
  
  // Calculate Day of Year
  DOY = date2doy(Local_Time[1], Local_Time[2], Local_Time[3], Local_Time[4], Local_Time[5]);
  
  LAT = LAT * M_PI / 180;
  
  
  A = (2*M_PI*(DOY-1)) / 365;
  DEC = b1 - b2 * cos(A) + b3 * sin(A) - b4 * cos(2*A) + b5 * sin(2*A) - b6 * cos(3*A) + b7 * sin(3*A);
  
  //printf("Solar Declination = %f\n", DEC);
  
  B =  (2*M_PI*(DOY-1)) / 365;
  ET = ((a1 + a2 * cos(B) - a3 * sin(B) - a4 * cos(2*B) - a5 * sin(2*B))*12) / M_PI;
  MST = UTC_Time + LNG / 15;
  TST = MST + ET;
  t = (15 * M_PI/180)*(TST-12);
  
  
  SZA_cos = cos(M_PI/2 - DEC)* cos(M_PI/2 - LAT) + sin(M_PI/2 - DEC) * sin(M_PI/2 - LAT) * cos(t);
  ANS = acos(SZA_cos) * 180.0/M_PI;
  
  //printf("%f\n", ANS);
  
  
  return ANS;
  
  
}//cal_sza


void date_errorcheck()
{

  // Last Day of the Month
  int Last_DD;


  // Year Check
  if(YYYY <= 0)
  {
     printf("Year %d is less than or equal to 0\n", YYYY);
     exit(1);

  }//fi

  // Month Check
  if(MM <= 0 || MM > 12)
  {
     printf("Month %d does not exist\n", MM);
     exit(1);

  }//fi

  // Day Check
  Last_DD = get_lastday(YYYY, MM);

  if(DD > Last_DD || DD <= 0)
  {
     printf("Date of %d%02d%02d does not exist\n", YYYY, MM, DD);
     exit(1);

  }//fi

  // Hour Check
  if(HH < 0 || HH >= 24)
  {
     printf("Hour %d does not exist\n", HH);
     exit(1);

  }//fi

  // Minute Check
  if(MN < 0 || MN >= 60)
  {
     printf("Minute %d does not exist\n", MN);
     exit(1);

  }//fi


  return;


}//date_errorcheck


void initialization()
{

  int I,J;


  read_formatfile();

  if((STN_LAT = (float *)malloc(sizeof(float)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed.\n"); exit(1);}
  if((STN_LNG = (float *)malloc(sizeof(float)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed.\n"); exit(1);}

  // STN_LAT/LNG Initialization
  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        STN_LAT[(I-1)*X+J] = 0;
        STN_LNG[(I-1)*X+J] = 0;

     }//J
  }//I
 
  read_standard_latlng();


  return;


}//initialization


void read_formatfile()
{

  int I;

  // Present Line
  int pline = 1;

  char filename[512];
  char readline[1000];
  char *param[100+1];


  FILE *fp;


  sprintf(filename, "config/format.txt");


  for(I=0;I<=100;I++)
  {
     if((param[I] = (char *)malloc(sizeof(char)*1000)) == NULL){ printf("malloc failed.\n"); exit(1);}

  }


  printf("%s  Loading ...", filename);

  fp = fopen(filename, "r");

  if(fp == NULL)
  {

     printf("%s read error\n", filename);
     exit(1);

  }//fi

  while( fgets(readline, 1000, fp) != NULL)
  {
     strcpy(param[pline], strtok(readline, ", "));
     pline++;

  }//while

  printf(" finish\n");

  fclose(fp);

  TIME_DIFF = atof(param[1]);   // int->doble for India 5.5H
  sprintf(TIME_Code, "%s", param[2]);
  X = atoi(param[3]);
  Y = atoi(param[4]);

  sprintf(ftype, "%s", param[5]);
  sprintf(input_DIR, "%s", param[6]);
  sprintf(Pickup_DIR, "%s", param[7]);
  sprintf(SL_DIR, "config/");
  sprintf(LATname, "%s", param[8]);
  sprintf(LNGname, "%s", param[9]);
  sprintf(ttype, "%s", param[10]);


  for(I=0;I<=100;I++) free(param[I]);

  return;


}//read_formatfile


void read_standard_latlng()
{

  // Filename of Latitude/Longitude Data
  char lat_filename[512];
  char lng_filename[512];

  sprintf(lat_filename, "%s%s", SL_DIR, LATname);
  sprintf(lng_filename, "%s%s", SL_DIR, LNGname);

  bin2f(lat_filename, X, Y, STN_LAT);
  bin2f(lng_filename, X, Y, STN_LNG);

  printf("  STN_LAT ...%f    STN_LNG ...%f\n", STN_LAT[100],STN_LNG[100]);

  return;


}//read_standard_latlng



void write_txt(char *filename, float *DD)
{

  FILE *fp;

  int I;


  printf("%s  Writing ...", filename);

  fp = fopen(filename, "w");

  if(fp == NULL)
  {
     printf("%s write error\n", filename);
     exit(1);

  }//fi

  fprintf(fp, "%7.4f %8.4f %6.3f %7.3f %9.4f %d %d %d %d %d %d %s\n", DD[1], DD[2], STN_LAT[(Pick_Pix[2]-1)*X+Pick_Pix[1]], STN_LNG[(Pick_Pix[2]-1)*X+Pick_Pix[1]], DD[3], Pick_Time_Local[1], Pick_Time_Local[2], Pick_Time_Local[3], Pick_Time_Local[4], Pick_Time_Local[5], Pick_Time_Local[6], TIME_Code);

    

  printf(" finish\n");

  fclose(fp);


  return;


}//write_txt
